@@ -1,6 +1,5 @@
 //! Custom cell impl
-
-use std::{cell::UnsafeCell, fmt, rc::Rc};
+use std::{cell::UnsafeCell, fmt, rc::Rc, rc::Weak};
 
 pub(super) struct Cell<T> {
     inner: Rc<UnsafeCell<T>>,
@@ -39,4 +38,28 @@ impl<T> Cell<T> {
     pub(super) fn get_mut(&self) -> &mut T {
         unsafe { &mut *self.inner.as_ref().get() }
     }
+
+    pub(super) fn downgrade(&self) -> WeakCell<T> {
+        WeakCell {
+            inner: Rc::downgrade(&self.inner),
+        }
+    }
+}
+
+pub(super) struct WeakCell<T> {
+    inner: Weak<UnsafeCell<T>>,
+}
+
+impl<T> Clone for WeakCell<T> {
+    fn clone(&self) -> Self {
+        Self {
+            inner: self.inner.clone(),
+        }
+    }
+}
+
+impl<T> WeakCell<T> {
+    pub(super) fn upgrade(&self) -> Option<Cell<T>> {
+        self.inner.upgrade().map(|inner| Cell { inner })
+    }
 }
